import vtk
import numpy as np
from numpy import zeros
import matplotlib.pyplot as plt
#from mpl_toolkits.mplot3d import Axes3D
#import matplotlib2tikz
from numpy import linalg as LA
import matplotlib.tri as mtri
import katoptron.utilities as u


def SaveFigureAsImage(fileName, fig=None, **kwargs):
    ''' Save a Matplotlib figure as an image without borders or frames.
       Args:
            fileName (str): String that ends in .png etc.

            fig (Matplotlib figure instance): figure you want to save as the image
        Keyword Args:
            orig_size (tuple): width, height of the original image used to maintain 
            aspect ratio.
    '''
    fig_size = fig.get_size_inches()
    w, h = fig_size[0], fig_size[1]
    fig.patch.set_alpha(0)
    if 'orig_size' in kwargs:  # Aspect ratio scaling if required
        w, h = kwargs['orig_size']
        w2, h2 = fig_size[0], fig_size[1]
        fig.set_size_inches([((w2 / w)) * w, ((w2 / w)) * h])
        fig.set_dpi(((w2 / w)) * fig.get_dpi())
    a = fig.gca()
    a.set_frame_on(False)
    a.set_xticks([])
    a.set_yticks([])
    plt.axis('off')
    plt.xlim(0, h)
    plt.ylim(w, 0)
    fig.savefig(fileName, transparent=True, bbox_inches='tight',
                pad_inches=0)


def compute_integrated_F(volume_id, component_id, cut_origin_x, cut_origin_y,
                         cut_origin_z, cut_normal_x, cut_normal_y,
                         cut_normal_z, reader):

    thresh = vtk.vtkThreshold()
    thresh.ThresholdBetween(volume_id - 0.5, volume_id + 0.5)
    thresh.SetInputConnection(reader.GetOutputPort())
    thresh.SetInputArrayToProcess(0, 0, 0,
                                  vtk.vtkDataObject.FIELD_ASSOCIATION_CELLS,
                                  "volume_id")
    thresh.Update()

    plane = vtk.vtkPlane()
    plane.SetOrigin(cut_origin_x, cut_origin_y, cut_origin_z)
    plane.SetNormal(cut_normal_x, cut_normal_y, cut_normal_z)

    cutter = vtk.vtkFiltersCorePython.vtkCutter()
    cutter.SetCutFunction(plane)
    cutter.SetInputConnection(thresh.GetOutputPort())
    cutter.Update()

    data = cutter.GetOutput()

    triangles = data.GetPolys().GetData()
    points = data.GetPoints()

    stress = data.GetCellData().GetArray('sigma')

    ntri = (triangles.GetNumberOfTuples() // 4)
    npts = points.GetNumberOfPoints()
    nstress = stress.GetNumberOfTuples()

    tri = zeros((ntri, 3), dtype=int)
    x = zeros(npts)
    y = zeros(npts)
    z = zeros(npts)

    sigma_vector = zeros(nstress)

    sigma_vector_node = zeros(npts)
    area_node = zeros(npts)

    for i in range(0, ntri):
        tri[i, 0] = triangles.GetTuple(4 * i + 1)[0]
        tri[i, 1] = triangles.GetTuple(4 * i + 2)[0]
        tri[i, 2] = triangles.GetTuple(4 * i + 3)[0]

    for i in range(npts):
        pt = points.GetPoint(i)
        x[i] = pt[0]
        y[i] = pt[1]
        z[i] = pt[2]

    for i in range(0, nstress):
        sigma = stress.GetTuple(i)
        sigma_vector[i] = sigma[component_id]

    fig = plt.figure(figsize=(8, 8))
    plt.set_cmap('coolwarm')
    #plt.triplot(x, y, tri)
    axtri = plt.tripcolor(x, y, tri, sigma_vector)
    #plt.scatter(x, y)
    plt.gca().set_aspect('equal')
    cb = fig.colorbar(axtri, orientation="horizontal")

    F = 0.

    for i in range(0, ntri):
        cross = np.cross(
            np.array([
                x[tri[i, 1]] - x[tri[i, 0]], y[tri[i, 1]] - y[tri[i, 0]],
                z[tri[i, 1]] - z[tri[i, 0]]
            ]),
            np.array([
                x[tri[i, 2]] - x[tri[i, 0]], y[tri[i, 2]] - y[tri[i, 0]],
                z[tri[i, 2]] - z[tri[i, 0]]
            ]))
        area = (LA.norm(cross) / 2)
        F = F + area * sigma_vector[i]
        for j in range(0, 3):
            sigma_vector_node[tri[i, j]] = sigma_vector_node[
                tri[i, j]] + area * sigma_vector[i]
            area_node[tri[i, j]] = area_node[tri[i, j]] + area

    for i in range(npts):
        sigma_vector_node[i] = (sigma_vector_node[i] / area_node[i])

    fig = plt.figure(figsize=(8, 8))
    plt.set_cmap('coolwarm')
    #plt.triplot(x, y, tri)
    axtri = plt.tricontourf(x, y, tri, sigma_vector_node)
    #plt.scatter(x, y)
    plt.gca().set_aspect('equal')
    cb = fig.colorbar(axtri, orientation="horizontal")

    return F


def post_process(
        filename='MIRROR_1_ASSEMBLY_fused_2_ast1_s0_0.vtu',
        filename_ref='MIRROR_1_ASSEMBLY_fused_2_ast1_s0_0.vtu',
        remove_ref=False,
        scale_u=1.,
        scale_T=1.,
        T_0=22.,
        p_cut=np.array(
            [-0.0156128890812397, 0.020816631615161896,
             0.0006999988108873367]),
        n_cut=np.array([1, 0, 0]),
        T_range=np.array([0., 0.]),
        dz_range=np.array([0., 0.]),
):
    if remove_ref:
        if filename_ref[-4:] == ".vtu":
            reader = vtk.vtkXMLUnstructuredGridReader()
        elif filename_ref[-5:] == ".pvtu":
            reader = vtk.vtkXMLPUnstructuredGridReader()

        reader.SetFileName(filename_ref)
        reader.Update()

        plane = vtk.vtkPlane()
        plane.SetOrigin(p_cut[0], p_cut[1], p_cut[2])
        plane.SetNormal(n_cut[0], n_cut[1], n_cut[2])

        theta = np.arctan(n_cut[1] * 1. / n_cut[0])

        cutter = vtk.vtkFiltersCorePython.vtkCutter()
        cutter.SetCutFunction(plane)
        cutter.SetInputConnection(reader.GetOutputPort())
        cutter.Update()

        data = cutter.GetOutput()

        triangles = data.GetPolys().GetData()
        points = data.GetPoints()

        mapper = vtk.vtkCellDataToPointData()
        mapper.AddInputData(data)
        mapper.Update()

        u = mapper.GetOutput().GetPointData().GetArray('displacement')

        ntri = (triangles.GetNumberOfTuples() // 4)
        npts = points.GetNumberOfPoints()

        dx_ref = zeros(npts)
        dy_ref = zeros(npts)
        dz_ref = zeros(npts)

        for i in range(0, npts):
            dx_ref[i] = scale_u * u.GetTuple(i)[0]
            dy_ref[i] = scale_u * u.GetTuple(i)[1]
            dz_ref[i] = scale_u * u.GetTuple(i)[2]

    if filename[-4:] == ".vtu":
        reader = vtk.vtkXMLUnstructuredGridReader()
    elif filename[-5:] == ".pvtu":
        reader = vtk.vtkXMLPUnstructuredGridReader()

    reader.SetFileName(filename)
    reader.Update()

    #data1 = reader.GetOutput()
    '''
    thresh = vtk.vtkThreshold()
    thresh.ThresholdBetween(8.5,11.5)
    thresh.SetInputConnection(reader.GetOutputPort())
    thresh.SetInputArrayToProcess(0, 0, 0, vtk.vtkDataObject.FIELD_ASSOCIATION_CELLS, "volume_id")
    thresh.Update()
    '''
    plane = vtk.vtkPlane()
    plane.SetOrigin(p_cut[0], p_cut[1], p_cut[2])
    plane.SetNormal(n_cut[0], n_cut[1], n_cut[2])

    theta = np.arctan(n_cut[1] * 1. / n_cut[0])

    cutter = vtk.vtkFiltersCorePython.vtkCutter()
    cutter.SetCutFunction(plane)
    cutter.SetInputConnection(reader.GetOutputPort())
    cutter.Update()

    data = cutter.GetOutput()

    triangles = data.GetPolys().GetData()
    points = data.GetPoints()

    mapper = vtk.vtkCellDataToPointData()
    mapper.AddInputData(data)
    mapper.Update()

    T = mapper.GetOutput().GetPointData().GetArray('T')
    u = mapper.GetOutput().GetPointData().GetArray('displacement')
    stress_VM_pt = mapper.GetOutput().GetPointData().GetArray('von Mises')

    stress = data.GetCellData().GetArray('sigma')
    stress_VM = data.GetCellData().GetArray('von Mises')
    volume_id = data.GetCellData().GetArray('volume_id')

    ntri = (triangles.GetNumberOfTuples() // 4)
    npts = points.GetNumberOfPoints()
    nvls = points.GetNumberOfPoints()  # T.GetNumberOfTuples()

    nstress = stress.GetNumberOfTuples()

    tri = zeros((ntri, 3), dtype=int)
    x = zeros(npts)
    y = zeros(npts)
    z = zeros(npts)
    dx = zeros(npts)
    dy = zeros(npts)
    dz = zeros(npts)
    T_vec = zeros(nvls)

    sigma_vector_node = zeros(npts)
    sigma_vector_node_filtered = zeros(npts)
    area_node = zeros(npts)

    sigma_zz = zeros(nstress)
    sigma_zz_filtered = zeros(nstress)
    volume_id_vec = zeros(nstress)
    volume_id_vec_2 = zeros(nstress)

    level_zz = np.linspace(-1e8, 1e8, 16)

    for i in range(0, ntri):
        tri[i, 0] = triangles.GetTuple(4 * i + 1)[0]
        tri[i, 1] = triangles.GetTuple(4 * i + 2)[0]
        tri[i, 2] = triangles.GetTuple(4 * i + 3)[0]

    for i in range(npts):
        pt = points.GetPoint(i)
        x[i] = scale_u * pt[0]
        y[i] = scale_u * pt[1]
        z[i] = scale_u * pt[2]

    for i in range(0, nvls):
        T_tmp = T.GetTuple(i)
        T_vec[i] = scale_T * T_tmp[0] + T_0

        dx[i] = scale_u * u.GetTuple(i)[0]
        dy[i] = scale_u * u.GetTuple(i)[1]
        dz[i] = scale_u * u.GetTuple(i)[2]

        sigma_vector_node_filtered[i] = scale_u**2 * stress_VM_pt.GetTuple(
            i)[0]

    if remove_ref:
        dx -= dx_ref
        dy -= dy_ref
        dz -= dz_ref

    print('T')
    print(np.amin(T_vec))
    print(np.amax(T_vec))

    print('u')
    print(np.amin(dx))
    print(np.amax(dx))
    print(np.amin(dy))
    print(np.amax(dy))
    print(np.amin(dz))
    print(np.amax(dz))

    for i in range(0, nstress):
        sigma = stress.GetTuple(i)
        sigma_zz[i] = scale_u**2 * sigma[8]

        volume_id_vec[i] = volume_id.GetTuple(i)[0]

        if sigma_zz[i] <= level_zz[0]:
            sigma_zz_filtered[i] = level_zz[0]
        elif sigma_zz[i] >= level_zz[15]:
            sigma_zz_filtered[i] = level_zz[15]
        else:
            sigma_zz_filtered[i] = sigma_zz[i]

    volume_id_vec_tmp = volume_id_vec

    volume_id_vec_tmp[np.where(volume_id_vec == 4)] = 3
    volume_id_vec_tmp[np.where(volume_id_vec == 5)] = 3

    volume_id_vec_tmp[np.where(volume_id_vec == 7)] = 6
    volume_id_vec_tmp[np.where(volume_id_vec == 8)] = 6

    volume_id_vec_tmp[np.where(volume_id_vec == 10)] = 9
    volume_id_vec_tmp[np.where(volume_id_vec == 11)] = 9

    volume_id_vec_tmp[np.where(volume_id_vec == 13)] = 12
    volume_id_vec_tmp[np.where(volume_id_vec == 14)] = 12

    volume_id_vec_tmp[np.where(volume_id_vec == 16)] = 15
    volume_id_vec_tmp[np.where(volume_id_vec == 17)] = 15

    volume_id_vec_tmp[np.where(volume_id_vec >= 19)] = 19
    #volume_id_vec_tmp[np.where( volume_id_vec ==17 )] = 15

    indices_volume_id = np.argsort(volume_id_vec_tmp)

    current_i = 0
    volume_id_vec_2[indices_volume_id[0]] = current_i
    for i in range(1, len(volume_id_vec)):
        if volume_id_vec_tmp[indices_volume_id[i - 1]] != volume_id_vec_tmp[
                indices_volume_id[i]]:
            current_i = current_i + 1
        volume_id_vec_2[indices_volume_id[i]] = current_i

    for i in range(0, ntri):
        cross = np.cross(
            np.array([
                x[tri[i, 1]] - x[tri[i, 0]], y[tri[i, 1]] - y[tri[i, 0]],
                z[tri[i, 1]] - z[tri[i, 0]]
            ]),
            np.array([
                x[tri[i, 2]] - x[tri[i, 0]], y[tri[i, 2]] - y[tri[i, 0]],
                z[tri[i, 2]] - z[tri[i, 0]]
            ]))
        area = (LA.norm(cross) / 2)
        for j in range(0, 3):
            sigma_vector_node[
                tri[i, j]] = sigma_vector_node[tri[i, j]] + area * sigma_zz[i]
            area_node[tri[i, j]] = area_node[tri[i, j]] + area

    for i in range(npts):
        sigma_vector_node[i] = (sigma_vector_node[i] / area_node[i])
        '''
        if sigma_vector_node[i] <= level_zz[0]:
            sigma_vector_node_filtered[i] = level_zz[0]
        elif sigma_vector_node[i] >= level_zz[15]:
            sigma_vector_node_filtered[i] = level_zz[15]
        else:
            sigma_vector_node_filtered[i] = sigma_vector_node[i]
        '''
    #tmp = stress.GetTuple(0)

    if T_range[0] == T_range[1]:
        T_range[0] = min(T_vec)
        T_range[1] = max(T_vec)
    level = np.linspace(T_range[0], T_range[1], 16)
    # Velocity x-component
    fig = plt.figure(figsize=(8, 8))
    plt.set_cmap('coolwarm')
    plt.tricontour(np.cos(theta) * y - np.sin(theta) * x,
                   -z,
                   tri,
                   T_vec,
                   level,
                   linestyles='-',
                   colors='black',
                   linewidths=0.5)
    axtri = plt.tricontourf(
        np.cos(theta) * y - np.sin(theta) * x, -z, tri, T_vec, level)
    f = open("T.txt", "w+")
    print(str(level[0]) + ', ' + str(level[3]) + ', ' + str(
        level[6]) + ', ' + str(level[9]) + ', ' + str(level[12]) + ', ' + str(
            level[15]), file=f)
    f.close()
    plt.gca().set_aspect('equal')
    plt.gca().set_axis_off()
    plt.subplots_adjust(top=1, bottom=0, right=1, left=0, hspace=0, wspace=0)
    plt.margins(0, 0)
    plt.gca().xaxis.set_major_locator(plt.NullLocator())
    plt.gca().yaxis.set_major_locator(plt.NullLocator())
    plt.savefig("T.pdf",
                transparent=True,
                bbox_inches='tight',
                pad_inches=-0.05)

    level = np.linspace(min(dx), max(dx), 16)
    # Velocity x-component
    fig = plt.figure(figsize=(8, 8))
    plt.set_cmap('coolwarm')
    plt.tricontour(np.cos(theta) * y - np.sin(theta) * x,
                   -z,
                   tri,
                   dx,
                   level,
                   linestyles='-',
                   colors='black',
                   linewidths=0.5)
    axtri = plt.tricontourf(
        np.cos(theta) * y - np.sin(theta) * x, -z, tri, dx, level)
    f = open("dx.txt", "w+")
    print(str(level[0]) + ', ' + str(level[3]) + ', ' + str(
        level[6]) + ', ' + str(level[9]) + ', ' + str(level[12]) + ', ' + str(
            level[15]), file=f)
    f.close()
    plt.gca().set_aspect('equal')
    plt.gca().set_axis_off()
    plt.subplots_adjust(top=1, bottom=0, right=1, left=0, hspace=0, wspace=0)
    plt.margins(0, 0)
    plt.gca().xaxis.set_major_locator(plt.NullLocator())
    plt.gca().yaxis.set_major_locator(plt.NullLocator())
    plt.savefig("dx.pdf",
                transparent=True,
                bbox_inches='tight',
                pad_inches=-0.05)

    level = np.linspace(min(dy), max(dy), 16)
    # Velocity x-component
    fig = plt.figure(figsize=(8, 8))
    plt.set_cmap('coolwarm')
    plt.tricontour(np.cos(theta) * y - np.sin(theta) * x,
                   -z,
                   tri,
                   dy,
                   level,
                   linestyles='-',
                   colors='black',
                   linewidths=0.5)
    axtri = plt.tricontourf(
        np.cos(theta) * y - np.sin(theta) * x, -z, tri, dy, level)
    f = open("dy.txt", "w+")
    print(str(level[0]) + ', ' + str(level[3]) + ', ' + str(
        level[6]) + ', ' + str(level[9]) + ', ' + str(level[12]) + ', ' + str(
            level[15]), file=f)
    f.close()
    plt.gca().set_aspect('equal')
    plt.gca().set_axis_off()
    plt.subplots_adjust(top=1, bottom=0, right=1, left=0, hspace=0, wspace=0)
    plt.margins(0, 0)
    plt.gca().xaxis.set_major_locator(plt.NullLocator())
    plt.gca().yaxis.set_major_locator(plt.NullLocator())
    plt.savefig("dy.pdf",
                transparent=True,
                bbox_inches='tight',
                pad_inches=-0.05)

    if dz_range[0] == dz_range[1]:
        dz_range[0] = min(dz)
        dz_range[1] = max(dz)
    level = np.linspace(dz_range[0], dz_range[1], 16)
    # Velocity x-component
    fig = plt.figure(figsize=(8, 8))
    plt.set_cmap('coolwarm')
    plt.tricontour(np.cos(theta) * y - np.sin(theta) * x,
                   -z,
                   tri,
                   dz,
                   level,
                   linestyles='-',
                   colors='black',
                   linewidths=0.5)
    axtri = plt.tricontourf(
        np.cos(theta) * y - np.sin(theta) * x, -z, tri, dz, level)
    f = open("dz.txt", "w+")
    print(str(level[0]) + ', ' + str(level[3]) + ', ' + str(
        level[6]) + ', ' + str(level[9]) + ', ' + str(level[12]) + ', ' + str(
            level[15]), file=f)
    f.close()
    plt.gca().set_aspect('equal')
    plt.gca().set_axis_off()
    plt.subplots_adjust(top=1, bottom=0, right=1, left=0, hspace=0, wspace=0)
    plt.margins(0, 0)
    plt.gca().xaxis.set_major_locator(plt.NullLocator())
    plt.gca().yaxis.set_major_locator(plt.NullLocator())
    plt.savefig("dz.pdf",
                transparent=True,
                bbox_inches='tight',
                pad_inches=-0.05)
    plt.close()
    '''
    #plt.minorticks_on()
    #plt.gca().set_xticklabels([])
    #plt.gca().set_yticklabels([])
    #plt.gca().set_axis_off()
    #plt.subplots_adjust(top = 1, bottom = 0, right = 1, left = 0, hspace = 0, wspace = 0)
    #plt.margins(0,0)
    #plt.gca().xaxis.set_major_locator(plt.NullLocator())
    #plt.gca().yaxis.set_major_locator(plt.NullLocator())
    #plt.axis('off')
    #plt.tricontour(x, y, tri, T_vec, 16)
    #plt.show
    #plt.savefig("T_vec.pdf", transparent=True, bbox_inches='tight', pad_inches=-0.05)

    #matplotlib2tikz.save('test.tex')

    cb = fig.colorbar(axtri, orientation="horizontal")

    fig = plt.figure(figsize=(8, 8))
    plt.set_cmap('coolwarm')
    axtri = plt.tripcolor(np.cos(theta)*y-np.sin(theta)*x, -z, tri, volume_id_vec_2)
    f = open("volume_id.txt", "w+")
    print >> f, str(level[0]) + ', ' + str(level[3]) + ', ' + str(level[6]) + ', ' + str(level[9]) + ', ' + str(level[12]) + ', ' + str(level[15])
    f.close()
    plt.gca().set_aspect('equal')
    plt.gca().set_axis_off()
    plt.subplots_adjust(top=1, bottom=0, right=1, left=0, hspace=0, wspace=0)
    plt.margins(0, 0)
    plt.gca().xaxis.set_major_locator(plt.NullLocator())
    plt.gca().yaxis.set_major_locator(plt.NullLocator())
    plt.savefig("volume_id.png",
                transparent=True,
                bbox_inches='tight',
                pad_inches=0.,
                dpi=2000)
    plt.savefig("volume_id.pdf",
                transparent=True,
                bbox_inches='tight',
                pad_inches=-0.05)

    linewidth = 2
    fig = plt.figure(figsize=(8, 8))
    plt.set_cmap('coolwarm')
    axtri = plt.tripcolor(np.cos(theta)*y-np.sin(theta)*x, -z, tri, volume_id_vec_2)
    plt.plot([0.00606, 0.01513], [0.018, 0.018], 'g-', lw=linewidth)
    plt.plot([0.03085, 0.03993], [0.018, 0.018], 'g-', lw=linewidth)
    plt.plot([0.04805, 0.05714], [0.018, 0.018], 'g-', lw=linewidth)
    plt.plot([0.07285, 0.08194], [0.018, 0.018], 'g-', lw=linewidth)

    plt.plot([0.00301297, 0.0862297], [-2.60656e-05, -2.60656e-05],
             'r-',
             lw=linewidth)

    plt.plot([0.01513, 0.01718, 0.01718], [0.018, 0.018, 0.007833],
             'r-',
             lw=linewidth)
    plt.plot([0.02881, 0.02881, 0.03085], [0.007833, 0.018, 0.018],
             'r-',
             lw=linewidth)
    plt.plot([0.05714, 0.0591934, 0.0591934], [0.018, 0.018, 0.007833],
             'r-',
             lw=linewidth)
    plt.plot([0.0708286, 0.0708286, 0.07285], [0.007833, 0.018, 0.018],
             'r-',
             lw=linewidth)

    plt.plot([0.00768668, 0.00768668, 0.0136152],
             [0.0299629, 0.028372, 0.028372],
             'r-',
             lw=linewidth)
    plt.plot([0.00768668, 0.0157517], [0.0314519, 0.0314519],
             'g-',
             lw=linewidth)
    plt.plot([0.0139112, 0.0163621], [0.0283628, 0.0283628],
             'g-',
             lw=linewidth)
    plt.plot([0.0139204, 0.0151413], [0.0253754, 0.0253754],
             'g-',
             lw=linewidth)
    plt.plot([0.0153632, 0.0170188], [0.0253754, 0.0253754],
             'g-',
             lw=linewidth)

    plt.plot([0.0118091, 0.0138181], [0.0519943, 0.0519943],
             'g-',
             lw=linewidth)

    plt.plot([0.0383327, 0.0383327, 0.0323929],
             [0.0299629, 0.028372, 0.028372],
             'r-',
             lw=linewidth)
    plt.plot([0.0302762, 0.0383327], [0.0314519, 0.0314519],
             'g-',
             lw=linewidth)
    plt.plot([0.0297374, 0.032108], [0.0283628, 0.0283628], 'g-', lw=linewidth)
    plt.plot([0.0289832, 0.0306937], [0.0253754, 0.0253754],
             'g-',
             lw=linewidth)
    plt.plot([0.0308688, 0.0321214], [0.0253754, 0.0253754],
             'g-',
             lw=linewidth)

    plt.plot([0.0322022, 0.0342226], [0.0519943, 0.0519943],
             'g-',
             lw=linewidth)

    plt.plot([0.0496977, 0.0496977, 0.0556013],
             [0.0299629, 0.028372, 0.028372],
             'r-',
             lw=linewidth)
    plt.plot([0.0496977, 0.0577345], [0.0314519, 0.0314519],
             'g-',
             lw=linewidth)
    plt.plot([0.0559981, 0.0583794], [0.0283628, 0.0283628],
             'g-',
             lw=linewidth)
    plt.plot([0.0559237, 0.0571888], [0.0253754, 0.0253754],
             'g-',
             lw=linewidth)
    plt.plot([0.0573872, 0.0590244], [0.0253754, 0.0253754],
             'g-',
             lw=linewidth)

    plt.plot([0.0538401, 0.0558245], [0.0519943, 0.0519943],
             'g-',
             lw=linewidth)

    plt.plot([0.0803319, 0.0803319, 0.0743539],
             [0.0299629, 0.028372, 0.028372],
             'r-',
             lw=linewidth)
    plt.plot([0.0722703, 0.0803815], [0.0314519, 0.0314519],
             'g-',
             lw=linewidth)
    plt.plot([0.0717246, 0.0741058], [0.0283628, 0.0283628],
             'g-',
             lw=linewidth)
    plt.plot([0.07103, 0.0726536], [0.0253754, 0.0253754], 'g-', lw=linewidth)
    plt.plot([0.0728542, 0.0741013], [0.0253754, 0.0253754],
             'g-',
             lw=linewidth)

    plt.plot([0.0742012, 0.0762138], [0.0519943, 0.0519943],
             'g-',
             lw=linewidth)

    plt.plot([0.0171631, 0.0171631], [0.0474981, 0.0542711],
             'r-',
             lw=linewidth)
    plt.plot([0.0288311, 0.0288311], [0.0474981, 0.0542711],
             'r-',
             lw=linewidth)
    plt.plot([0.0591792, 0.0591792], [0.0474981, 0.0542711],
             'r-',
             lw=linewidth)
    plt.plot([0.0708312, 0.0708312], [0.0474981, 0.0542711],
             'r-',
             lw=linewidth)

    f = open("volume_id_with_contact.txt", "w+")
    print >> f, str(level[0]) + ', ' + str(level[3]) + ', ' + str(level[6]) + ', ' + str(level[9]) + ', ' + str(level[12]) + ', ' + str(level[15])
    f.close()
    plt.gca().set_aspect('equal')
    plt.gca().set_axis_off()
    plt.subplots_adjust(top=1, bottom=0, right=1, left=0, hspace=0, wspace=0)
    plt.margins(0, 0)
    plt.gca().xaxis.set_major_locator(plt.NullLocator())
    plt.gca().yaxis.set_major_locator(plt.NullLocator())
    plt.savefig("volume_id_with_contact.png",
                transparent=True,
                bbox_inches='tight',
                pad_inches=0.,
                dpi=2000)
    plt.savefig("volume_id_with_contact.pdf",
                transparent=True,
                bbox_inches='tight',
                pad_inches=-0.05)

    #plt.show()

    fig = plt.figure(figsize=(8, 8))
    plt.set_cmap('coolwarm')
    axtri = plt.tripcolor(np.cos(theta)*(y+ 100 * dy)-np.sin(theta)*(x+ 100 * dx), -z - 100 * dz, tri, volume_id_vec_2)    
    f = open("volume_id_def.txt", "w+")
    print >> f, str(level[0]) + ', ' + str(level[3]) + ', ' + str(level[6]) + ', ' + str(level[9]) + ', ' + str(level[12]) + ', ' + str(level[15])
    f.close()
    plt.gca().set_aspect('equal')
    plt.gca().set_axis_off()
    plt.subplots_adjust(top=1, bottom=0, right=1, left=0, hspace=0, wspace=0)
    plt.margins(0, 0)
    plt.gca().xaxis.set_major_locator(plt.NullLocator())
    plt.gca().yaxis.set_major_locator(plt.NullLocator())
    plt.savefig("volume_id_def.png",
                transparent=True,
                bbox_inches='tight',
                pad_inches=0.,
                dpi=2000)
    plt.savefig("volume_id_def.pdf",
                transparent=True,
                bbox_inches='tight',
                pad_inches=-0.05)

    level = np.linspace(-1e8, 1e8, 16)
    fig = plt.figure(figsize=(8, 8))
    plt.set_cmap('coolwarm')
    axtri = plt.tripcolor(np.cos(theta)*y-np.sin(theta)*x, -z, tri, sigma_zz, level)    
    f = open("sigma_zz_unsmooth_2.txt", "w+")
    print >> f, str(level[0]) + ', ' + str(level[3]) + ', ' + str(level[6]) + ', ' + str(level[9]) + ', ' + str(level[12]) + ', ' + str(level[15])
    f.close()
    plt.gca().set_aspect('equal')
    plt.gca().set_axis_off()
    plt.subplots_adjust(top=1, bottom=0, right=1, left=0, hspace=0, wspace=0)
    plt.margins(0, 0)
    plt.gca().xaxis.set_major_locator(plt.NullLocator())
    plt.gca().yaxis.set_major_locator(plt.NullLocator())
    plt.savefig("sigma_zz_unsmooth_2.pdf",
                transparent=True,
                bbox_inches='tight',
                pad_inches=-0.05)

    level = np.linspace(-1e8, 1e8, 16)
    fig = plt.figure(figsize=(8, 8))
    plt.set_cmap('coolwarm')
    axtri = plt.tripcolor(np.cos(theta)*y-np.sin(theta)*x, -z, tri, sigma_zz_filtered, level)    
    f = open("sigma_zz_filtered.txt", "w+")
    print >> f, str(level[0]) + ', ' + str(level[3]) + ', ' + str(level[6]) + ', ' + str(level[9]) + ', ' + str(level[12]) + ', ' + str(level[15])
    f.close()
    plt.gca().set_aspect('equal')
    plt.gca().set_axis_off()
    plt.subplots_adjust(top=1, bottom=0, right=1, left=0, hspace=0, wspace=0)
    plt.margins(0, 0)
    plt.gca().xaxis.set_major_locator(plt.NullLocator())
    plt.gca().yaxis.set_major_locator(plt.NullLocator())
    plt.savefig("sigma_zz_filtered.pdf",
                transparent=True,
                bbox_inches='tight',
                pad_inches=-0.05)

    #cb = fig.colorbar(axtri, orientation="horizontal")

    level = np.linspace(-2e8, 2e8, 16)
    fig = plt.figure(figsize=(8, 8))
    plt.set_cmap('coolwarm')
    axtri = plt.tricontourf(np.cos(theta)*y-np.sin(theta)*x, -z, tri, sigma_vector_node, level)    
    f = open("sigma_zz_smooth_2.txt", "w+")
    print >> f, str(level[0]) + ', ' + str(level[3]) + ', ' + str(level[6]) + ', ' + str(level[9]) + ', ' + str(level[12]) + ', ' + str(level[15])
    f.close()

    plt.gca().set_aspect('equal')
    plt.gca().set_axis_off()
    plt.subplots_adjust(top=1, bottom=0, right=1, left=0, hspace=0, wspace=0)
    plt.margins(0, 0)
    plt.gca().xaxis.set_major_locator(plt.NullLocator())
    plt.gca().yaxis.set_major_locator(plt.NullLocator())
    plt.savefig("sigma_zz_smooth_2.pdf",
                transparent=True,
                bbox_inches='tight',
                pad_inches=-0.05)

    level = np.linspace(min(sigma_vector_node_filtered),
                        max(sigma_vector_node_filtered), 16)
    fig = plt.figure(figsize=(8, 8))
    plt.set_cmap('coolwarm')
    axtri = plt.tricontourf(np.cos(theta)*y-np.sin(theta)*x, -z, tri, sigma_vector_node_filtered, level)

    f = open("sigma_vector_node_filtered.txt", "w+")
    print >> f, str(level[0]) + ', ' + str(level[3]) + ', ' + str(level[6]) + ', ' + str(level[9]) + ', ' + str(level[12]) + ', ' + str(level[15])
    f.close()
    plt.gca().set_aspect('equal')
    plt.gca().set_axis_off()
    plt.subplots_adjust(top=1, bottom=0, right=1, left=0, hspace=0, wspace=0)
    plt.margins(0, 0)
    plt.gca().xaxis.set_major_locator(plt.NullLocator())
    plt.gca().yaxis.set_major_locator(plt.NullLocator())
    plt.savefig("sigma_vector_node_filtered.pdf",
                transparent=True,
                bbox_inches='tight',
                pad_inches=-0.05)

    #cb = fig.colorbar(axtri, orientation="horizontal")


    #plt.show()
    '''


def post_process_mirror(filename='mirror_data.npz',
                        filename_ref='mirror_data.npz',
                        remove_ref=False,
                        round_lim=True):
    if remove_ref:
        npzfile = np.load(filename_ref)
        dz_ref = npzfile['dz'].flatten()

    npzfile = np.load(filename)

    x = npzfile['x']
    y = npzfile['y']
    z = npzfile['z']
    T = npzfile['T'].flatten()
    tri = npzfile['tri']
    dz = npzfile['dz'].flatten()

    if remove_ref:
        dz -= dz_ref

    tri2 = mtri.Triangulation(x, y, triangles=tri)

    f = open("mirror_surface_levels.txt", "w+")

    OAX, OAY = 0.0595, 0.017

    optical_coefficients, dz_residual = u.compute_RBM_Curvature_Irreg(
        x, y, z, tri, dz, "m", np.array([OAX, OAY]))
    np.savez('mirror_optical_results',
             optical_coefficients=optical_coefficients,
             dz_residual=dz_residual)
    np.savetxt('mirror_optical_results.out', optical_coefficients)
    '''
    level = np.linspace(min(dx), max(dx), 16)
    # Velocity x-component
    fig = plt.figure(figsize=(8, 8))
    plt.set_cmap('coolwarm')
    plt.tricontour(x,
                   y,
                   tri,
                   dx,
                   level,
                   linestyles='-',
                   colors='black',
                   linewidths=0.5)
    axtri = plt.tricontourf(x, y, tri, dx, level)    
    print >> f, "dx"
    print >> f, level * 1000
    plt.gca().set_aspect('equal')
    plt.gca().set_axis_off()
    plt.subplots_adjust(top=1, bottom=0, right=1, left=0, hspace=0, wspace=0)
    plt.margins(0, 0)
    plt.gca().xaxis.set_major_locator(plt.NullLocator())
    plt.gca().yaxis.set_major_locator(plt.NullLocator())
    plt.savefig("dx_m.pdf",
                transparent=True,
                bbox_inches='tight',
                pad_inches=-0.05)

    level = np.linspace(min(dy), max(dy), 16)
    # Velocity x-component
    fig = plt.figure(figsize=(8, 8))
    plt.set_cmap('coolwarm')
    plt.tricontour(x,
                   y,
                   tri,
                   dy,
                   level,
                   linestyles='-',
                   colors='black',
                   linewidths=0.5)
    axtri = plt.tricontourf(x, y, tri, dy, level)
    print >> f, "dy"
    print >> f, level * 1000
    plt.gca().set_aspect('equal')
    plt.gca().set_axis_off()
    plt.subplots_adjust(top=1, bottom=0, right=1, left=0, hspace=0, wspace=0)
    plt.margins(0, 0)
    plt.gca().xaxis.set_major_locator(plt.NullLocator())
    plt.gca().yaxis.set_major_locator(plt.NullLocator())
    plt.savefig("dy_m.pdf",
                transparent=True,
                bbox_inches='tight',
                pad_inches=-0.05)
    '''
    if round_lim:
        min_T = min(T)
        max_T = max(T)
    else:
        min_T = min(T)
        max_T = max(T)
    level = np.linspace(min_T, max_T, 16)
    # Velocity x-component
    fig = plt.figure(figsize=(8, 8))
    plt.set_cmap('coolwarm')
    plt.tricontour(tri2,
                   T,
                   level,
                   linestyles='-',
                   colors='black',
                   linewidths=0.5)
    axtri = plt.tricontourf(tri2, T, level)
    print("T", file=f)
    print(level, file=f)
    plt.gca().set_aspect('equal')
    plt.gca().set_axis_off()
    plt.subplots_adjust(top=1, bottom=0, right=1, left=0, hspace=0, wspace=0)
    plt.margins(0, 0)
    plt.gca().xaxis.set_major_locator(plt.NullLocator())
    plt.gca().yaxis.set_major_locator(plt.NullLocator())
    plt.savefig("T.pdf",
                transparent=True,
                bbox_inches='tight',
                pad_inches=-0.05)

    round_dz = 0.000005
    if round_lim:
        min_dz = np.floor((min(dz) / round_dz)) * round_dz
        max_dz = np.ceil((max(dz) / round_dz)) * round_dz
    else:
        min_dz = min(dz)
        max_dz = max(dz)

    level = np.linspace(min_dz, max_dz, 17)
    # Velocity x-component
    fig = plt.figure(figsize=(8, 8))
    plt.set_cmap('coolwarm')
    plt.tricontour(tri2,
                   dz,
                   level,
                   linestyles='-',
                   colors='black',
                   linewidths=0.5)
    axtri = plt.tricontourf(tri2, dz, level)
    print("dz", file=f)
    print(level * 1000, file=f)
    plt.gca().set_aspect('equal')
    plt.gca().set_axis_off()
    plt.subplots_adjust(top=1, bottom=0, right=1, left=0, hspace=0, wspace=0)
    plt.margins(0, 0)
    plt.gca().xaxis.set_major_locator(plt.NullLocator())
    plt.gca().yaxis.set_major_locator(plt.NullLocator())
    plt.savefig("dz_m.pdf",
                transparent=True,
                bbox_inches='tight',
                pad_inches=-0.05)

    dz = 1000. * dz
    #level = np.linspace(1000*min_dz, 1000*max_dz, 16)
    # Velocity x-component
    fig = plt.figure(figsize=(8, 8))
    plt.set_cmap('coolwarm')
    plt.tricontour(tri2,
                   dz,
                   1000. * level,
                   linestyles='-',
                   colors='black',
                   linewidths=0.5)
    axtri = plt.tricontourf(tri2, dz, 1000. * level)
    print("dz", file=f)
    print(level * 1000, file=f)
    plt.gca().set_aspect('equal')
    plt.gca().set_axis_off()
    plt.subplots_adjust(top=1, bottom=0, right=1, left=0, hspace=0, wspace=0)
    plt.margins(0, 0)
    plt.gca().xaxis.set_major_locator(plt.NullLocator())
    plt.gca().yaxis.set_major_locator(plt.NullLocator())
    cb = fig.colorbar(axtri, orientation="horizontal")
    cb.set_label('Normal displacement [mm]', rotation=0)
    plt.savefig("dz_m_2.pdf",
                transparent=True,
                bbox_inches='tight',
                pad_inches=0.05)
    f.close()
