namespace katoptron
{

/**
 * @brief Algebraic constructor
 * 
 * This contrusctor calls the constructor of the katoptron::Graph, katoptron::Matrices, and katoptron::Vectors classes.
 * To do so, we need information regarding the distributions of the nodes and elements over the MPI processes (information
 * included in the katoptron::Map class) and information regarding the connectivity between the degrees of freedom (or,
 * equivalently, between the nodes) (information included in katoptron::Domain::elementsList).
 *
 * Arguments:
 * <ul>
 *  <li> map: an RCP to a Map object,
 *  <li> domain: an RCP to a Domain object.
 * </ul>
 */
template <typename scalar>
Algebraic<scalar>::Algebraic(Teuchos::RCP<Map> _map, Teuchos::RCP<Domain<scalar>> domain)
{
    map = _map;

    LinearSolver::getTimers()["graph"].start();
    graph = Teuchos::rcp(new Graph(_map, domain->elementsList));
    LinearSolver::getTimers()["graph"].stop();
    matrices = Teuchos::rcp(new Matrices<scalar>(graph));
    vectors = Teuchos::rcp(new Vectors<scalar>(map));
}

/**
 * @brief Function to read a vector from file and distribute it to the MPI processes.
 * 
 * Arguments:
 * <ul>
 *  <li> name: the name of input file,
 *  <li> myRank: rank of the process.
 * </ul>
 */
template <typename scalar>
Teuchos::RCP<Tpetra::Vector<scalar, Map::local_ordinal_type, Map::global_ordinal_type>>
Algebraic<scalar>::readVectorFromFile(std::string &name, size_t myRank)
{
    using Teuchos::RCP;
    using Teuchos::rcp;

    std::ifstream inputfile;
    if (myRank == 0)
        inputfile.open(name);

    RCP<Tpetra::Vector<scalar, Map::local_ordinal_type, Map::global_ordinal_type>> x_0 =
        rcp(new Tpetra::Vector<scalar, Map::local_ordinal_type, Map::global_ordinal_type>(map->mapOutput, true));
    RCP<Tpetra::Vector<scalar, Map::local_ordinal_type, Map::global_ordinal_type>> x =
        rcp(new Tpetra::Vector<scalar, Map::local_ordinal_type, Map::global_ordinal_type>(map->mapDofs, true));

    if (myRank == 0)
    {
        auto line_size = map->mapOutput->getGlobalNumElements();

        typedef EnsembleTraits<scalar> ET;
        const int ensemble_size = ET::size;

        for (Map::global_ordinal_type global_index = 0; global_index < line_size; ++global_index)
        {
            scalar tmp;
            for (auto j = 0; j < ensemble_size; ++j)
                inputfile >> ET::coeff(tmp, j);

            x_0->replaceGlobalValue(global_index, tmp);
        }
        inputfile.close();
    }

    Tpetra::Export<> exportDof(map->mapOutput, map->mapDofs);

    x->doExport(*x_0, exportDof, Tpetra::ADD);
    return x;
}

}; // namespace katoptron
